/// <reference types="@cloudflare/workers-types" />
/**
 * Config for the parser
 *
 * @property {boolean | string[]} allowSurrogateDelegation - Allows Surrogate Delegation
 * If boolean and the Request has valid Surrogate Delegation headers then no parsing will take place and requests will be sent onwards
 * If Array of strings then each string is treated as an IP Address. If the originally connecting IP matches one of those IPs then Delegation will happen
 * @property {string[]} contentTypes - Array of strings of content types that the parser should parse for ESI Tags
 * Note: That these are case sensitive. See - https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Type
 * @property {boolean} disableThirdPartyIncludes - Whether or not to enable third party includes (includes from other domains)
 * @property {number} [recursionLimit] - Levels of recusion the parser is allowed to go do
 * think includes that include themselves causing recusion
 * @default 10
 * @property {string[]} thirdPatyIncludesDomainWhitelist - If third party includes are disabled you can white list them by including domains here
 * @property {string[]} varsCookieBlacklist - Array of strings of cookies that will be blacklisted from being expanded in esi VARs.
 */
export declare type ESIConfig = {
    allowSurrogateDelegation?: boolean | string[];
    contentTypes?: string[];
    disableThirdPartyIncludes?: boolean;
    recursionLimit?: number;
    thirdPatyIncludesDomainWhitelist?: string[];
    varsCookieBlacklist?: string[];
};
export declare type ESIVars = {
    headers: {
        [key: string]: string;
    };
    method: string;
    esiArgs: URLSearchParams;
    url: URL;
};
/**
 * ESI Event Data for the Current Request
 *
 * @property {ESIConfig} config - ESIConfig when class was created
 * @property {Object} headers - All headers of the request uppercased
 * @property {string} method - Method of the request
 * @property {URLSearchParams} esiArgs - Any ESI Arguments extracted from the URL Search params of the original request
 * Will be a URLSearchParam encoded object
 * @property {customESIVars} customVars - If a custom ESI Vars function is supplied this will be the result of that function
 * @property {URL} url - URL Object of the Request with ESI Args removed
 * @property {Request} request - Request object after ESI Args have been removed
 * @property {number} recursion - Recusion level we're currently at
 */
export declare type ESIEventData = {
    /**
     * {ESIConfig} for the current Request
     */
    config: ESIConfig;
    /**
     * All headers of the current Request in {Object}
     * All headers are uppercassed with - being converted to _
     */
    headers: {
        [key: string]: string;
    };
    /**
     * Request Method
     */
    method: string;
    /**
     * Any ESI Arguments
     */
    esiArgs: URLSearchParams;
    /**
     * If a custom ESI Vars function is supplied this will be the result of that function
     *
     * @default false
     */
    customVars?: customESIVars;
    /**
     * {URL} Object of the Request with any ESI Args removed
     */
    url: URL;
    /**
     * Mutatable {Request} object with the ESI Args removed
     */
    request: Request;
    /**
     * Level of recursion the function is currently at
     *
     * @default 0
     */
    recursion: number;
};
export declare type customESIVars = {
    [key: string]: string | {
        [key: string]: string;
    };
};
export declare type customESIVarsFunction = (request: Request) => Promise<customESIVars>;
export declare type fetchFunction = (request: string | Request, requestInitr?: Request | RequestInit | undefined) => Promise<Response>;
export declare class esi {
    options: ESIConfig;
    esiFunction?: customESIVarsFunction;
    fetcher: fetchFunction;
    constructor(options?: ESIConfig, customESIFunction?: customESIVarsFunction, fetcher?: typeof fetch);
    parse(origRequest: Request, recursion?: number): Promise<Response>;
    handleESI(eventData: ESIEventData, text: string): Promise<string>;
    handleTEXT(eventData: ESIEventData, text: string): Promise<string>;
    streamBody(eventData: ESIEventData, readable: ReadableStream, writable: WritableStream): Promise<void>;
    validContentType(response: Response): boolean;
    validSurrogateControl(response: Response): boolean;
}
export declare const esiArgsPrefix = "esi_";
/**
 * Returns Processor Token string for Surrogate headers
 * eg "ESI"
 *
 * @returns {string} - supported procesor token
 */
export declare function getProcessorToken(): string;
/**
 * Returns Processor Version as a number for Surrogate headers
 * eg 1.0
 *
 * @returns {number} - processor supported version
 */
export declare function getProcessorVersion(): number;
/**
 * Returns Processor Version as a string for Surrogate headers
 * eg "1.0"
 *
 * @returns {string} - processor supported version
 */
export declare function getProcessorVersionString(): string;
export default esi;
